#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include <Adafruit_MPU6050.h>
#include <Adafruit_Sensor.h>
#include <Wire.h>
#include "model.h" // tutaj podajemy wygenerowany wcześniej model
#define TRUNCATE 20 // ograniczenie odczytów do +/- 20
#define ACCEL_THRESHOLD 5 // próg uruchamiania detekcji
#define NUM_SAMPLES 100 // liczba odczytywanych próbek
#define NUM_AXES 6 // trzy osie akcelerometru + trzy żyroskopu 

#define SCREEN_WIDTH 128 // szerokość wyświetlacza OLED
#define SCREEN_HEIGHT 32 // wysokość wyświetlacza OLED 
#define OLED_RESET     -1 // pin resetu wyświetlacza OLED

float features[NUM_AXES * NUM_SAMPLES]; // przechowuje gest przed detekcją 
int INT_PIN  = 0; // przerwanie od przycisku 
volatile boolean StartDetection = false; // moment detekcji 
double baseline[NUM_AXES]; // wykorzystywane w kalibracji

Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);
Adafruit_MPU6050 mpu;

Eloquent::ML::Port::RandomForest classifier; // klasa we wnętrzu model.h

void ICACHE_RAM_ATTR IntCallback() { // Upewnij się, że to przerwanie jest powyżej funkcji setup
  StartDetection = true;
}

void calibrate(){
    sensors_event_t a, g, temp;
    Serial.println("Calibrating...");
    testdrawchar("Calibrating...");
    for (int i = 0; i < 10; i++) {
        mpu.getEvent(&a, &g, &temp);
        delay(100);
    }
     Serial.println("done Calibrating!");
     testdrawchar("done Calibrating...");
    baseline[0] = double(a.acceleration.x);
    baseline[1] = double(a.acceleration.y);
    baseline[2] = double(a.acceleration.z); 
    baseline[3] = double(g.acceleration.z); 
    baseline[4] = double(g.acceleration.z); 
    baseline[5] = double(g.acceleration.z);  
    testdrawchar("Do some gesture!");
}

void recordIMU() { // Funkcja nagrywające dane z sensora po wykryciu ruchu
    float ax, ay, az;
    float gx, gy, gz;
    sensors_event_t a, g, temp;
    for (int i = 0; i < NUM_SAMPLES; i++) {
         mpu.getEvent(&a, &g, &temp);
  
    ax = constrain(a.acceleration.x - baseline[0], -TRUNCATE, TRUNCATE);
    ay = constrain(a.acceleration.y - baseline[1], -TRUNCATE, TRUNCATE);
    az = constrain(a.acceleration.z - baseline[2], -TRUNCATE, TRUNCATE);
    gx = constrain(g.acceleration.x - baseline[0], -TRUNCATE, TRUNCATE);
    gy = constrain(g.acceleration.y - baseline[1], -TRUNCATE, TRUNCATE);
    gz = constrain(g.acceleration.z - baseline[2], -TRUNCATE, TRUNCATE);

    int index = i*NUM_AXES;
    features[index] = ax;
    features[index+1] = ay;
    features[index+2] = az;
    features[index+3] = gx;
    features[index+4] = gy;
    features[index+5] = gz;  
    }
}

bool motionDetected(float ax, float ay, float az) { // Funkcja wykrywająca ruch
    return (abs(ax) + abs(ay) + abs(az)) > ACCEL_THRESHOLD;
}

void testdrawchar(String text) {
  display.clearDisplay();
  display.setTextSize(1);      
  display.setTextColor(SSD1306_WHITE);
  display.setCursor(0, 0);     
  display.print(text);
  display.display();
  delay(1000);
}

void classify() { // Funkcja realizująca klasyfikację
    Serial.print("Detected gesture: ");
    String gesture = classifier.predictLabel(features);
    Serial.println(gesture);
    testdrawchar(gesture);
    delay(2000);
    testdrawchar("Do some gesture!");
}

void setup() {
  Serial.begin(115200);
  pinMode(INT_PIN, INPUT);
  attachInterrupt(digitalPinToInterrupt(INT_PIN), IntCallback, FALLING);
  if(!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) { 
    Serial.println(F("SSD1306 allocation failed"));
    for(;;); 
  }
  while(!StartDetection)
  {
    delay(10);
  }
  testdrawchar("booting...");
  if (!mpu.begin()) {
    Serial.println("Failed to find MPU6050 chip");
    while (1) {
      delay(10);
    }
  }
  mpu.setAccelerometerRange(MPU6050_RANGE_4_G);
  mpu.setGyroRange(MPU6050_RANGE_500_DEG);
  mpu.setFilterBandwidth(MPU6050_BAND_21_HZ);
  Serial.println("");
  delay(100);
  calibrate();
}

void loop() { // Główna pętla programu
    float ax, ay, az; 
    sensors_event_t a, g, temp;
    mpu.getEvent(&a, &g, &temp);
  
    ax = constrain(a.acceleration.x - baseline[0], -TRUNCATE, TRUNCATE);
    ay = constrain(a.acceleration.y - baseline[1], -TRUNCATE, TRUNCATE);
    az = constrain(a.acceleration.z - baseline[2], -TRUNCATE, TRUNCATE);

    if (!motionDetected(ax, ay, az)) {
        delay(10);
        return;
    }
    Serial.println("Motion detected!");
      recordIMU();
      classify(); 
}
